#!/usr/bin/env python

"""Upload.py: Sends file to COM port."""
__author__      = "Wayne Parham"

import os, serial, sys, time

port        = "COM3"                   # Can be set on command line as arg[2]
rate        =  4800                    # Can be set on command line as arg[3]
bytesize    =  serial.SEVENBITS        # Can be set on command line as arg[4]
parity      =  serial.PARITY_NONE      # Can be set on command line as arg[5]
stopbits    =  serial.STOPBITS_ONE     # Can be set on command line as arg[6]
verbose     =  True
char_delay  =  0.05
line_delay  =  0.5

if( len(sys.argv) < 2 ):
   sys.exit( "Error: missing filename argument" )

filename    = str(sys.argv[1])

if( len(sys.argv) > 2 ):
   port = str(sys.argv[2])
if( len(sys.argv) > 3 ):
   rate = str(sys.argv[3])
if( len(sys.argv) > 4 ):
   if( str(sys.argv[4]) == "8" ):
      bytesize = serial.EIGHTBITS
   else:
      bytesize = serial.SEVENBITS
if( len(sys.argv) > 5 ):
   if( str(sys.argv[5].upper()) == "E" ):
      parity = serial.PARITY_EVEN
   elif( str(sys.argv[5].upper()) == "O" ):
      parity = serial.PARITY_ODD
   elif( str(sys.argv[5].upper()) == "M" ):
      parity = serial.PARITY_MARK
   elif( str(sys.argv[5].upper()) == "S" ):
      parity = serial.PARITY_SPACE
   else:
      parity = serial.PARITY_NONE
if( len(sys.argv) > 6 ):
   if( str(sys.argv[6]) == "2" ):
      stopbits = serial.STOPBITS_TWO
   else:
      stopbits = serial.STOPBITS_ONE

if( os.path.isfile(filename) ):

   try:
      ser = serial.Serial(
         port     = port,
         baudrate = rate,
         bytesize = bytesize,
         parity   = parity,
         stopbits = stopbits,
         timeout  = 1
      )

      print( "\nUploading " + filename + " to " + port + "." ) 

      if verbose:
         print

      with open( filename ) as fileobj:
         for line in fileobj:
            if verbose:
               print( line.rstrip('\n') )
            for char in line: 
               ser.write(char)
               if( char_delay > 0 ):
                  time.sleep( char_delay )
            ser.write( b'\r' )
            if( line_delay > 0 ):
               time.sleep( line_delay )
         ser.write( b'\r' )

   except:
      print( "\n" + port + " is in use.  Release it and try again." )

else:
   print( "\n" + filename + " is locked or does not exist." )

