#!/usr/bin/env python

"""Upload_hex.py: Sends file to COM port."""
__author__      = "Wayne Parham"

import os, serial, sys, time

port        = "COM3"                   # Can be set on command line as arg[2]
rate        =  4800                    # Can be set on command line as arg[3]
bytesize    =  serial.SEVENBITS        # Can be set on command line as arg[4]
parity      =  serial.PARITY_NONE      # Can be set on command line as arg[5]
stopbits    =  serial.STOPBITS_ONE     # Can be set on command line as arg[6]
verbose     =  True
char_delay  =  0.01
byte_delay  =  0.1
char_count  =  0
filename = str(sys.argv[1])

hex_digits      = 'a','A','b','B','c','C','d','D','e','E','f','F', \
                  '0','1','2','3','4','5','6','7','8','9' 

if( len(sys.argv) < 2 ):
   sys.exit( "Error: missing filename argument" )

filename    = str(sys.argv[1])

if( len(sys.argv) > 2 ):
   port = str(sys.argv[2])
if( len(sys.argv) > 3 ):
   rate = str(sys.argv[3])
if( len(sys.argv) > 4 ):
   if( str(sys.argv[4]) == "8" ):
      bytesize = serial.EIGHTBITS
   else:
      bytesize = serial.SEVENBITS
if( len(sys.argv) > 5 ):
   if( str(sys.argv[5].upper()) == "E" ):
      parity = serial.PARITY_EVEN
   elif( str(sys.argv[5].upper()) == "O" ):
      parity = serial.PARITY_ODD
   elif( str(sys.argv[5].upper()) == "M" ):
      parity = serial.PARITY_MARK
   elif( str(sys.argv[5].upper()) == "S" ):
      parity = serial.PARITY_SPACE
   else:
      parity = serial.PARITY_NONE
if( len(sys.argv) > 6 ):
   if( str(sys.argv[6]) == "2" ):
      stopbits = serial.STOPBITS_TWO
   else:
      stopbits = serial.STOPBITS_ONE

if( os.path.isfile(filename) ):

   try:
      ser = serial.Serial(
         port     = port,
         baudrate = rate,
         bytesize = bytesize,
         parity   = parity,
         stopbits = stopbits,
         timeout  = 1
      )

      print( "\nUploading " + filename + " to " + port + "." ) 

      if verbose:
         print

      with open( filename ) as fileobj:
         for line in fileobj:
            for char in line:
               if( char in hex_digits ):
                  char_count += 1
                  ser.write(char)
                  if verbose:
                     sys.stdout.write( char )
                  if( (char_delay > 0) and     (char_count % 2) ):
                     time.sleep( char_delay )
                  if( (byte_delay > 0) and not (char_count % 2) ):
                     time.sleep( byte_delay )

   except:
      print( "\n" + port + " is in use.  Release it and try again." )

else:
   print( "\n" + filename + " is locked or does not exist." )
